# [[file:monticello-drive.org::*Nix flake encapsulation][Nix flake encapsulation:1]]
{
  inputs = {
    ergogen = {
      url = "git+https://code.rix.si/rrix/ergogen";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    bosl2 = {
      url = "github:BelfrySCAD/BOSL2";
      flake = false;
    };
    zmk-nix = {
      url = "github:lilyinstarlight/zmk-nix";
      inputs.nixpkgs.follows = "nixpkgs";
    };
  };

  outputs = { self, nixpkgs, zmk-nix, ... }@inputs: let
    forAllSystems = nixpkgs.lib.genAttrs (nixpkgs.lib.attrNames inputs.zmk-nix.packages);
    localSource = nixpkgs.lib.sourceFilesBySuffices self
      [
        # zmk files
        ".conf" ".keymap" ".yml" ".overlay" ".shield" ".defconfig"
        # openscad files
        ".scad"
        # py script
        ".py"
        # ergogen yaml
        ".yaml"
      ];
  in {
    packages = forAllSystems (system: 
      let
        pkgs = import nixpkgs { inherit system; };
      in rec {
        default = firmware;
        ergogen = inputs.ergogen.packages.${system}.default;
        kicad_pcb = pkgs.writeScriptBin "kicad_pcb" ''
          ${ergogen}/bin/ergogen ./ergogen.yaml -o ./
        '';
        stls = let
          mkOpenSCAD = 
            variant: (import ./nix/openscad.nix { inherit pkgs; }) {
              name = "atreoid-${variant}";
              src = localSource;
              scadLibs = { BOSL2 = inputs.bosl2; };
              scadFiles = [ "./scad/atreoid-${variant}.scad" ];
            };
        in pkgs.symlinkJoin {
          name = "stls";
          paths = [
            (mkOpenSCAD "left")
            (mkOpenSCAD "right")
            (mkOpenSCAD "center")
          ];
        };
        firmware = zmk-nix.legacyPackages.${system}.buildKeyboard {
          name = "firmware";
        
          src = nixpkgs.lib.trace localSource.outPath localSource;
        
          board = "nice_nano_v2";
          shield = "monticello_drive";
          config = "zmk-config";
        
          zephyrDepsHash = "sha256-JakmFT5VDxL3KoOTXSzM3kvE6aqbMVRIHovveKN7OdA=";
        
          meta = {
            description = "ZMK firmware";
            license = nixpkgs.lib.licenses.mit;
            platforms = nixpkgs.lib.platforms.all;
          };
        };
        
        flash = zmk-nix.packages.${system}.flash.override { inherit firmware; };
        update = zmk-nix.packages.${system}.update;
   });

    devShells = forAllSystems (system: let
      pkgs = import nixpkgs { inherit system; };
      pyEnv = pkgs.python3.withPackages (pp: with pp; [
        click
        pyyaml
      ]);
      zmk-shell = inputs.zmk-nix.devShells.${system}.default;
    in {
      default = pkgs.mkShell {
        inputsFrom = [ zmk-shell ];
        packages = with pkgs; [
          pyEnv
        ];
      };
    });
  };
}
# Nix flake encapsulation:1 ends here
