# [[file:../monticello-drive.org::*STLs][STLs:3]]
# greetz pjones
{ pkgs ? import <nixpkgs> { }
}:

{
  # Source code directory, usually `./'.  Passed to `mkDerivation'.
  src

# List of files to process.  The default is an empty list which means
# to process all `*.scad' files.
, scadFiles ? [ ]
# File variables to define
, defines ? { }
# Add SCAD libraries to openscad PATH
, scadLibs ? {}

# The name of the derivation.
, name ? "project"
}:
let
  lib = pkgs.lib;

  scadLibPath = pkgs.stdenv.mkDerivation {
    name = "scad-paths";
    inherit src;

    installPhase = ''
      mkdir -p $out
      ${pkgs.lib.concatStrings
        (pkgs.lib.mapAttrsToList
          (name: path: "cp -r ${path} $out/${name}\n") scadLibs)}
    '';
  };

  buildScript =
    pkgs.writeShellScript
      "openscad-build" ''
      #!/usr/bin/env bash
      
      set -eu
      set -o pipefail
      
      files=()
      if [ $# -eq 0 ]; then
        readarray -t files < <(find . -type f -name '*.scad')
      else
        files=("$@")
      fi
      
      for file in "''${files[@]}"; do
        echo "==> STL: $file"
        export OPENSCADPATH=${scadLibPath} 
        echo $OPENSCADPATH
        openscad \
          ${pkgs.lib.concatStrings
            (pkgs.lib.mapAttrsToList
              (var: val: "-D${var}=${val} ")
              defines)} \
          -o "$(dirname "$file")/$(basename "$file" .scad).stl" \
          "$file"
      done
    '';

  installScript =
    pkgs.writeShellScript
    "openscad-install" ''
      #!/usr/bin/env bash
      
      set -eu
      set -o pipefail
      
      out=$1
      ext=$2
      
      files=()
      mkdir -p "$out"
      
      readarray -t files < <(find . -type f -name "*.$ext")
      
      for file in "''${files[@]}"; do
        echo $file $out
        install -m 0444 "$file" "$out"
      done
    '';
in
pkgs.stdenvNoCC.mkDerivation {
  inherit src;
  name = "openscad-${name}";

  buildInputs = with pkgs; [
    openscad
  ];

  buildPhase = ''
    ${buildScript} ${lib.escapeShellArgs scadFiles}
  '';

  installPhase = ''
    ${installScript} "$out/stl" stl
  '';
}
# STLs:3 ends here
