include <BOSL2/std.scad>
include <BOSL2/screws.scad>
include <BOSL2/hinges.scad>

$fn = 24;

half="left";
// $center_w = 0;
$center_w = 25;

$plate_depth = 3.3;
$base_joiner_h = 10;
$half_w = 155;
$half_h = 140;

$wire_d = 2;
mcu_width = 18;
mcu_length = 33;
mcu_height = 4.25;  // Distance to top of PCB
mcu_row_spacing = 15.24;
mcu_pin_count = 24;
mcu_pin_pitch = 2.54;
mcu_pin_offset = 0;  // Offset from the rear of the PCB
mcu_connector_width = 13;  // Width of the connector (for plate cutout)
mcu_connector_length = 4;  // Distance the connector extends onto the MCU (for plate cutout)
mcu_connector_height = 8;  // Height of the plug housing
mcu_connector_offset = 2; // Vertical offset of plug center from PCB center
mcu_pcb_thickness = 1.6;
mcu_socket_width = mcu_width+4;
mcu_socket_length = mcu_length+4;
grid_size = 8;
grid_spacing = 1.6;

include <points.scad>

module hs_mcu(borders=[0,0,0,0]) {
  attachable(anchor=BOTTOM, size=[mcu_socket_width, mcu_socket_length, 8]) {
    translate([0,-17,-4]) 
      union() {
      difference() {
        // Socket base
        translate([-mcu_socket_width/2,-2,0])
          cube([mcu_socket_width,mcu_socket_length,$plate_depth+mcu_pcb_thickness]);

        // Wire cutouts
        for (row = [-1,1]) {
          for (pin = [0:mcu_pin_count/2-1]) {
            translate([
                       row*(mcu_row_spacing/2-1),
                       (pin+0.5)*mcu_pin_pitch,
                       $plate_depth-2/3
                       ]) rotate([0,row*90,0])
              cylinder(h=1000,d=2);
          }
        }
        // MCU cutout
        translate([-mcu_width/2,0,$plate_depth])
          cube([mcu_width, mcu_length,mcu_pcb_thickness+1]);
        // Side cutout
        translate([-(mcu_socket_width+2)/2,0,$plate_depth])
          cube([mcu_socket_width+2,mcu_pin_count/2*mcu_pin_pitch,mcu_pcb_thickness+1]);
        // USB cutout
        translate([-mcu_connector_width/2,-3,$plate_depth])
          cube([mcu_connector_width,mcu_socket_length+2,mcu_pcb_thickness+1]);
        translate([-mcu_connector_width/2,mcu_length-mcu_connector_length,$plate_depth-2])
          cube([mcu_connector_width,mcu_connector_length+3,$plate_depth+1]);

        // Relief to let you pop the MCU out
        translate([0,0,$plate_depth-1])
          cylinder(h=mcu_pcb_thickness+2,d=mcu_connector_width);
        translate([-mcu_connector_width/2,-3,$plate_depth-1])
          cube([mcu_connector_width,3,mcu_pcb_thickness+2]);
      }

      // Retention Tabs
      for (x = [-1,1]) {
        translate([x*(mcu_width+mcu_connector_width)/4,0,($plate_depth+mcu_pcb_thickness+1)/2]) {
          for (y = [-1,mcu_length+1]) {
            translate([0,y,0])
              cube(
                   [(mcu_width-mcu_connector_width)/2,2,$plate_depth+mcu_pcb_thickness+1],
                   center=true
                   );
          }
        }
        translate([x*(mcu_width+mcu_connector_width)/4,0,$plate_depth+mcu_pcb_thickness+0.5]) {
          rotate([0,90,0]) {
            for (y = [0,mcu_length]) {
              translate([0,y,0])
                cylinder(h=(mcu_width-mcu_connector_width)/2,d=1,center=true);
            }
          }
        }
      };
    }
    children();
  }
}

module center_plate() {
  diff() cube([$center_w,$half_h,$plate_depth], anchor=BOTTOM) {
    fwd(10) position(RIGHT+BACK) attach(BOTTOM, overlap=1) right_hinge();
    fwd(10) position(LEFT+BACK) attach(BOTTOM, overlap=1) left_hinge();
    back(10) position(LEFT+FRONT) attach(BOTTOM, overlap=1) zrot(180) right_hinge();
    back(10) position(RIGHT+FRONT) attach(BOTTOM, overlap=1) zrot(180) left_hinge();

    attach(BOTTOM, overlap=$plate_depth-$wire_d) back(20) hs_mcu();
    tag("remove") fwd(45) attach(BOTTOM, overlap=$plate_depth+1) cuboid([10,3,10]);
  }
}

module right_hand() {
  diff() cube([$half_w,$half_h,$plate_depth], anchor=BOTTOM) {
    right(5) position(FWD) xdistribute(l=$half_h) {
      corner_mount(width=5, anchor=BOTTOM+LEFT+BACK);
      corner_mount(width=5, anchor=BOTTOM+BACK);
      corner_mount(width=5, anchor=BOTTOM+BACK);
      corner_mount(width=5, anchor=BOTTOM+RIGHT+BACK);
    }
    right(5) position(BACK) xdistribute(l=$half_h) {
      corner_mount(width=5, anchor=BOTTOM+LEFT+FWD);
      corner_mount(width=5, anchor=BOTTOM+FWD);
      corner_mount(width=5, anchor=BOTTOM+FWD);
      corner_mount(width=5, anchor=BOTTOM+RIGHT+FWD);
    }
    fwd(10) position(LEFT+BACK) attach(BOTTOM, overlap=1) left_hinge();
    back(10) position(LEFT+FRONT) attach(BOTTOM, overlap=1) zrot(180) right_hinge();

    // woo magic numbers..
    down($plate_depth*1.7) attach(TOP, TOP) tag("remove") keys();
    attach(TOP, BOTTOM) left(38) back(30) zrot(10) tag("remove") track_cutout();
  }
}

module left_hand() {
  diff() cube([$half_w,$half_h,$plate_depth], anchor=BOTTOM) {
    left(5) position(FWD) xdistribute(l=$half_h) {
      corner_mount(width=5, anchor=BOTTOM+LEFT+BACK);
      corner_mount(width=5, anchor=BOTTOM+BACK);
      corner_mount(width=5, anchor=BOTTOM+BACK);
      corner_mount(width=5, anchor=BOTTOM+RIGHT+BACK);
    }
    left(5) position(BACK) xdistribute(l=$half_h) {
      corner_mount(width=5, anchor=BOTTOM+LEFT+FWD);
      corner_mount(width=5, anchor=BOTTOM+FWD);
      corner_mount(width=5, anchor=BOTTOM+FWD);
      corner_mount(width=5, anchor=BOTTOM+RIGHT+FWD);
    }
    fwd(10) position(RIGHT+BACK) attach(BOTTOM, overlap=1) right_hinge();
    back(10) position(RIGHT+FRONT) attach(BOTTOM, overlap=1) zrot(180) left_hinge();

    down($plate_depth*1.7) attach(TOP, TOP) tag("remove") xflip() keys();
  }
}

module keyboard_plate() {
  hide("test")
    union() {
    // right hand
    if(half == 0 || half == "right")
      recolor("red") right_hand();

    if ($center_w > 0 && half <= 0){
      left($half_w/2+$center_w/2)
        center_plate();
    }

    if(half == 0 || half == "left")
      // left hand
      recolor("blue")
        left($half_w+$center_w) left_hand();
  }
}

module corner_mount(width=5, anchor) {
  render() xrot(180)
    up(1.6)
    cube([width, width, 1], anchor=anchor)
    attach(TOP, BOTTOM) cyl(h=$base_joiner_h-1, d1=width, d2=width*0.8, anchor=BOTTOM, rounding2=1);
}

module track_cutout() {
  render() attachable(size=[23,19.6,8]) {
    down(3+$plate_depth) zrot(180)
      cuboid([23.0, 12.4, 3]) {
      attach(FRONT, BACK) cuboid([13.4, 7.4, 3]);
      attach(TOP, BOTTOM) fwd(1) xdistribute(l=19) {
        cyl(d=2, h=5);
        cyl(d=6.5+2, h=5);
        cyl(d=2, h=5);
      }
    };
    children();
  }
}

module keys() {
  render() attachable(anchor=BOTTOM, size=[131,110, 3.3]) {
    // cuboid([131,110, 3.3]);
    recolor("blue") left(71) fwd(32) down(3.3)
      union() for (tuple = array) {
      translate([tuple[0], tuple[1],0]) {
        zrot(tuple[2])
          cuboid([14,14,1.4])
          attach(TOP, BOTTOM)
          cuboid([15,15,2.1]);
      }
    };
    children();
  }
}

// module bolt_block(len=$base_joiner_h*2) {
//   xrot(90) render() // diff()
//     cuboid([$base_joiner_h, $base_joiner_h, len],
//            rounding=$base_joiner_h*0.3,
//            edges=[BACK, TOP, BOTTOM],
//            anchor=FRONT, orient=RIGHT)
//     screw_hole("M3", length=len, head="socket", counterbore=6, anchor=CENTER)
//     position(BOT) nut_trap_inline(l=6,anchor=BOT);
// }
module left_hinge() {
  zrot(-90) knuckle_hinge($base_joiner_h*2, offset=4, segs=3, inner=true, pin_diam=2);
}

module right_hinge() {
  zrot(90) knuckle_hinge($base_joiner_h*2, offset=4, segs=3, pin_diam=2);
}
